"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.serviceCtlMixin = exports.ServiceCtl = void 0;
const xstate_1 = require("xstate");
const version_js_1 = require("../version.js");
const machine_config_js_1 = require("./machine-config.js");
const machine_options_js_1 = require("./machine-options.js");
const wait_for_selector_js_1 = require("./wait-for-selector.js");
const guard_machine_event_js_1 = require("./guard-machine-event.js");
class EmptyClass {
}
const serviceCtlMixin = (superClass) => {
    class ServiceCtlMixin extends superClass {
        constructor(...args) {
            super(...args);
            const machineOptions = (0, machine_options_js_1.buildMachineOptions)({
                // reset: () => has been internally implemented by calling stop() and start()
                start: () => this.onStart(),
                stop: () => this.onStop(),
            });
            const machine = (0, xstate_1.createMachine)(machine_config_js_1.config, machineOptions);
            this._service = (0, xstate_1.interpret)(machine);
            this._service.start();
        }
        start() {
            (0, guard_machine_event_js_1.guardMachineEvent)(this._service, 'START');
            const started = (0, wait_for_selector_js_1.waitForMachineState)(this._service, 'active');
            const canceled = (0, wait_for_selector_js_1.waitForMachineState)(this._service, 'canceled');
            this._service.send('START');
            return Promise.race([
                started,
                canceled,
            ]);
        }
        stop() {
            (0, guard_machine_event_js_1.guardMachineEvent)(this._service, 'STOP');
            const stopped = (0, wait_for_selector_js_1.waitForMachineState)(this._service, 'inactive');
            const canceled = (0, wait_for_selector_js_1.waitForMachineState)(this._service, 'canceled');
            this._service.send('STOP');
            return Promise.race([
                stopped,
                canceled,
            ]);
        }
        reset() {
            (0, guard_machine_event_js_1.guardMachineEvent)(this._service, 'RESET');
            const started = (0, wait_for_selector_js_1.waitForMachineState)(this._service, 'active');
            const canceled = (0, wait_for_selector_js_1.waitForMachineState)(this._service, 'canceled');
            this._service.send('RESET');
            return Promise.race([
                started,
                canceled,
            ]);
        }
    }
    ServiceCtlMixin.VERSION = version_js_1.VERSION;
    return ServiceCtlMixin;
};
exports.serviceCtlMixin = serviceCtlMixin;
class ServiceCtl extends serviceCtlMixin(EmptyClass) {
}
exports.ServiceCtl = ServiceCtl;
//# sourceMappingURL=service-ctl.js.map