import { createMachine, interpret, } from 'xstate';
import { VERSION } from '../version.js';
import { config, } from './machine-config.js';
import { buildMachineOptions } from './machine-options.js';
import { waitForMachineState } from './wait-for-selector.js';
import { guardMachineEvent } from './guard-machine-event.js';
class EmptyClass {
}
const serviceCtlMixin = (superClass) => {
    class ServiceCtlMixin extends superClass {
        constructor(...args) {
            super(...args);
            const machineOptions = buildMachineOptions({
                // reset: () => has been internally implemented by calling stop() and start()
                start: () => this.onStart(),
                stop: () => this.onStop(),
            });
            const machine = createMachine(config, machineOptions);
            this._service = interpret(machine);
            this._service.start();
        }
        start() {
            guardMachineEvent(this._service, 'START');
            const started = waitForMachineState(this._service, 'active');
            const canceled = waitForMachineState(this._service, 'canceled');
            this._service.send('START');
            return Promise.race([
                started,
                canceled,
            ]);
        }
        stop() {
            guardMachineEvent(this._service, 'STOP');
            const stopped = waitForMachineState(this._service, 'inactive');
            const canceled = waitForMachineState(this._service, 'canceled');
            this._service.send('STOP');
            return Promise.race([
                stopped,
                canceled,
            ]);
        }
        reset() {
            guardMachineEvent(this._service, 'RESET');
            const started = waitForMachineState(this._service, 'active');
            const canceled = waitForMachineState(this._service, 'canceled');
            this._service.send('RESET');
            return Promise.race([
                started,
                canceled,
            ]);
        }
    }
    ServiceCtlMixin.VERSION = VERSION;
    return ServiceCtlMixin;
};
class ServiceCtl extends serviceCtlMixin(EmptyClass) {
}
export { ServiceCtl, serviceCtlMixin, };
//# sourceMappingURL=service-ctl.js.map